/*********************************************************************/
/* file: files.c - funtions for logfile and reading/writing comfiles */
/*                             HUT TINTIN                            */
/*                coded by Ossi Tulonen from TinTin III              */
/*                TinTin III coded by peter unold 1992               */
/*********************************************************************/
/*
 * TinTin III code Copyright (c) 1992 Peter Unold
 * All changed code from TinTin III Copyright (c) 1992 1993 1994 Ossi tulonen
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
 
#include <stdio.h>
#include <ctype.h>
#ifndef NSTDLIBS
  #include <stdlib.h>
#endif
#include <string.h>

#include "structs.h"
#include "color.h"
#include "files.h"
#include "llist.h"
#include "parse.h"
#include "screen.h"
#include "utility.h"

extern struct master_table_data master_table;
extern unsigned char tintin_char;
extern unsigned char text_start, text_end;

/* ***********************************
 * Open tintin file to read or write *
 *********************************** */


FILE *open_file(const unsigned char *name, const unsigned char *mode,
		session *ses)
{
  unsigned char buffer[BUFFER_SIZE];
  FILE *fl = 0;

  if(*name == '~' && *(name + 1) == '/') {
    if(getenv("HOME")) {
      sprintf((char *)buffer, "%s/%s", getenv("HOME"), name + 2);
      fl = fopen((const char *)buffer, (const char *)mode);
    } else {
      ses->print((const uchar *)"#Couldn't get your home dir.\n\r");
      return 0;
    }
  } else if(*name == '.' && *(name + 1) == '/') {
    name += 2;
    fl = fopen((const char *)name, (const char *)mode);
  } else if(*name != '/') {
    if(getenv("TINTINDIR")) {
      sprintf((char *)buffer, "%s/%s", getenv("TINTINDIR"), name);
    } else if(getenv("HOME")) {
      sprintf((char *)buffer, "%s/.tintin/%s", getenv("HOME"), name);
    } else {
      ses->print((const uchar *)"#Couldn't get your home dir, using current dir\n\r");
      buffer[0] = '\0';
    }
    if(!*buffer || !(fl = fopen((const char *)buffer, (const char *)mode)))
      fl = fopen((const char *)name, (const char *)mode);
  }
  
  return(fl);
}

/********************/
/* the #log command */
/********************/
session *log_command(const unsigned char *arg, session *ses)
{
  if(!ses->logfile) {
    if(*arg) { 
      if((ses->logfile = open_file(arg, (const uchar *)"w", ses)))
	ses->print((const uchar *)"#OK. LOGING.....\n\r");
      else
	ses->print((const uchar *)"#COULDN'T OPEN FILE.\n\r");
    } else
      ses->print((const uchar *)"#SPECIFY A FILENAME.\n\r");
    
  } else {
    fclose(ses->logfile);
    ses->logfile = NULL;
    ses->print((const uchar *)"#OK. LOGGING TURNED OFF.\n\r");
  }
  return(ses);
}

session *comm_logline(const unsigned char *arg, session *ses)
{
  unsigned char name[BUFFER_SIZE], buf[BUFFER_SIZE];
  FILE *fl;

  arg = get_arg_stop_spaces(arg, name);
  arg = get_arg_with_spaces(arg, buf);
  
  if(*name && *buf) {
    if((fl = open_file(name, (const uchar *)"a", ses))) {
      fprintf(fl, "%s: %s\n", own_asctime(), buf);
      fclose(fl);
      ses->print((const uchar *)"#OK.\n\r");
    } else {
      ses->print((const uchar *)"#COULDN'T OPEN FILE.\n\r");
    }
  } else {
    ses->print((const uchar *)"#Usage: logline <file name> <logged line>\n\r");
  }
  return(ses);
}

session *comm_logseshist(const unsigned char *arg, session *ses)
{
  unsigned char name[BUFFER_SIZE], numbuf[BUFFER_SIZE];
  int number, pos;
  FILE *fl;

  arg = get_arg_stop_spaces(arg, name);
  arg = get_arg_with_spaces(arg, numbuf);
  if(*numbuf) {
    number = string_to_number(numbuf, ses);
    if(number < 0)
      number = 0;
    else if(number > ses->get_hist_num())
      number = ses->get_hist_num();
  } else {
    number = ses->get_hist_num();
  }
  
  if(*name) {
    if((fl = open_file(name, (const uchar *)"a", ses))) {
      pos = ses->get_hist_end() - number;
      if(pos < 0)
	pos += MAX_SES_HIST;
      
      for(; number > 0; number--, pos++) {
	if(pos == MAX_SES_HIST)
	  pos = 0;
	fprintf(fl, "%s\n", ses->get_scrhistory(pos));
      }
      fclose(fl);
      ses->print((const uchar *)"#OK.\n\r");
    } else {
      ses->print((const uchar *)"#COULDN'T OPEN FILE.\n\r");
    }
  } else {
    ses->print((const uchar *)"#Usage: logseshist <file name> [lines number]\n\r");
  }
  return(ses);
}

// *******************************************
// session history autologin when session dies
// *******************************************
session *comm_autolog(const unsigned char *arg, session *ses)
{
  unsigned char buf[BUFFER_SIZE];
  unsigned char name[BUFFER_SIZE], numbuf[BUFFER_SIZE];
  
  arg = get_arg_stop_spaces(arg, name);
  arg = get_arg_with_spaces(arg, numbuf);
  
  if(*name) {
    if(*numbuf) {
      sprintf((char *)buf, "[%s] [%s]", name, numbuf);
    } else {
      sprintf((char *)buf, "[%s]", name);
    }
    ses->set_autolog(buf);
    ses->print((const uchar *)"#This session historys autolog is now on\n\r");
  } else {
    ses->set_autolog(0);
    ses->print((const uchar *)"#This session historys autolog is now off\n\r");
  }
  return(ses);
}

/***********************************/
/* read and execute a command file */
/***********************************/
session *read_command(const unsigned char *filename, session *ses)
{
  unsigned char buffer[BUFFER_SIZE], line[BUFFER_SIZE], *result;
  const unsigned char *linepos;
  int pos, inside;
  session *sestmp;
  FILE *myfile;

  if(filename) {
    if((myfile = open_file(filename, (const uchar *)"rt", ses))==NULL) {
      sprintf((char *)line,
	      "#Error - Couldn't open '%s' named file\n\r", filename);
      ses->print(line);
      return ses;
    }
  } else {
    sprintf((char *)buffer, "%s/.tintinrc", getenv("HOME"));
    if((myfile = fopen((const char *)buffer, "rt"))==NULL) {
      return ses;
    }
  }
  sestmp = ses;
  result = (uchar *)fgets((char *)line, sizeof(line), myfile);
  while(result) {
    if(!master_table.verbatimmode) {
      inside = 0;
      pos = 0;

      do {
	while(add_str(buffer, pos, sizeof(buffer), inside, line, TRUE, ses) &&
	      fgets((char *)line, sizeof(line), myfile))
	  ;
	
	result = (uchar *)fgets((char *)line, sizeof(line), myfile);
	linepos = space_out(line);
      } while(*linepos == text_start);

      *(buffer + pos) = '\0';

      if(!*buffer)
	continue;
      
      if(!(sestmp = parse_input(buffer, ses)) && ses)
	break;
    } else {
      if(!(sestmp = parse_input(line, ses)) && ses)
	break;
      result = (uchar *)fgets((char *)line, sizeof(line), myfile);
    }
  }
  
  fclose(myfile);
  return sestmp;
}

/************************/
/* write a command file */
/************************/

void write_word_list(FILE *fl, const int all, const session *ses)
{
  unsigned char buf[BUFFER_SIZE], tmp[BUFFER_SIZE];
  int i, k, pos;

  
  *buf = '\0';
  pos = 0;
  for(i = 0, k = 0; i < WORD_LIST_SIZE && ses->word_list[i]; i++) {
    if(all || !master_table.main_session->search_word(ses->word_list[i])) {
      if(k == 4) {
	fprintf(fl, "%cword%s %c%s%c\n",
		tintin_char, buf,
		text_start, ses->word_list[i], text_end);
	*buf = '\0';
	pos = 0;
	k = 0;
      } else {
	sprintf((char *)tmp,
		" %c%s%c", text_start, ses->word_list[i], text_end);
	pos = str_cat(buf, pos, sizeof(buf), tmp);
	k++;
      }
    }
  }
  if(k) {
    fprintf(fl, "%cword%s\n", tintin_char, buf);
  }
}

session *write_command(const unsigned char *filename, session *ses)
{
  unsigned char buf[BUFFER_SIZE];
  FILE *myfile;
  struct listnode *nodeptr;

  if((myfile = open_file(filename, (const uchar *)"w", ses)) == NULL) {
    ses->print((const uchar *)"#ERROR - COULDN'T OPEN THAT FILE.\n\r");
    return(ses);
  }

  if(tintin_char != DEFAULT_TINTIN_CHAR)
    fprintf(myfile, "%cchar %c\n", DEFAULT_TINTIN_CHAR, tintin_char);

  if(text_start != DEFAULT_TEX_START_CHAR || text_end != DEFAULT_TEX_END_CHAR)
    fprintf(myfile, "%ctext %c%c\n", tintin_char, text_start, text_end);

  nodeptr = ses->aliases;
  while(nodeptr) {
    fprintf(myfile, "%calias %c%s%c %c%s%c\n",
            tintin_char,
            text_start, nodeptr->left, text_end,
            text_start, nodeptr->right, text_end);
    nodeptr = nodeptr->next;
  }

  nodeptr = ses->actions;
  while(nodeptr) {
    if(nodeptr->type == TYPE_ACTION) {
      fprintf(myfile, "%caction %c%s%c %c%s%c %c%d%c\n",
	      tintin_char,
	      text_start, nodeptr->left, text_end,
	      text_start, nodeptr->right, text_end,
	      text_start, nodeptr->pri, text_end);
    } else {
      fprintf(myfile, "%csubstitute %c%s%c %c%s%c %c%d%c\n",
	      tintin_char,
	      text_start, nodeptr->left, text_end,
	      text_start, nodeptr->right, text_end,
	      text_start, nodeptr->pri, text_end);
    }
    nodeptr = nodeptr->next;
  }

  nodeptr = ses->colors;
  while(nodeptr) {
    color_to_text(nodeptr->right, buf, FALSE);
    fprintf(myfile, "%ccolor %c%s%c %c%s%c\n",
            tintin_char,
            text_start, nodeptr->left, text_end,
            text_start, buf, text_end);
    nodeptr = nodeptr->next;
  }

  nodeptr = ses->variables;
  while(nodeptr) {
    fprintf(myfile, "%cvariable %c%s%c %c%s%c\n",
            tintin_char,
            text_start, nodeptr->left, text_end,
            text_start, nodeptr->right, text_end);
    nodeptr = nodeptr->next;
  }

  write_word_list(myfile, TRUE, ses);

  fclose(myfile);
  ses->print((const uchar *)"#COMMANDO-FILE WRITTEN.\n\r");
  return ses;
}

/************************/
/* write a command file */
/************************/
session *writesession_command(const unsigned char *filename,
				     session *ses)
{
  unsigned char buf[BUFFER_SIZE];
  FILE *myfile;
  struct listnode *nodeptr;
  
  if((myfile = open_file(filename, (const uchar *)"w", ses)) == NULL) {
    ses->print((const uchar *)"#ERROR - COULDN'T OPEN THAT FILE.\n\r");
    return(ses);
  }

  if(tintin_char != DEFAULT_TINTIN_CHAR)
    fprintf(myfile, "%cchar %c\n", DEFAULT_TINTIN_CHAR, tintin_char);

  if(text_start != DEFAULT_TEX_START_CHAR || text_end != DEFAULT_TEX_END_CHAR)
    fprintf(myfile, "%ctext %c%c\n", tintin_char, text_start, text_end);

  nodeptr= ses->aliases;
  while(nodeptr) {
    if(ses && searchnode_list(master_table.main_session->aliases,
			      nodeptr->left)) {
      nodeptr = nodeptr->next;
      continue;
    }
    fprintf(myfile, "%calias %c%s%c %c%s%c\n",
            tintin_char,
            text_start, nodeptr->left, text_end,
            text_start, nodeptr->right, text_end);
    nodeptr = nodeptr->next;
  }

  nodeptr = ses->actions;
  while(nodeptr) {
    if(ses && searchnode_list(master_table.main_session->actions,
			      nodeptr->left)) {
      nodeptr = nodeptr->next;
      continue;
    }
    if(nodeptr->type == TYPE_ACTION) {
      fprintf(myfile, "%caction %c%s%c %c%s%c %c%d%c\n",
	      tintin_char,
	      text_start, nodeptr->left, text_end,
	      text_start, nodeptr->right, text_end,
	      text_start, nodeptr->pri, text_end);
    } else {
      fprintf(myfile, "%csubstitute %c%s%c %c%s%c %c%d%c\n",
	      tintin_char,
	      text_start, nodeptr->left, text_end,
	      text_start, nodeptr->right, text_end,
	      text_start, nodeptr->pri, text_end);
    }
    nodeptr = nodeptr->next;
  }

  nodeptr = ses->colors;
  while(nodeptr) {
    if(ses && searchnode_list(master_table.main_session->colors,
			      nodeptr->left)) {
      nodeptr = nodeptr->next;
      continue;
    }
    color_to_text(nodeptr->right, buf, FALSE);
    fprintf(myfile, "%ccolor %c%s%c %c%s%c\n",
            tintin_char,
            text_start, nodeptr->left, text_end,
            text_start, buf, text_end);
    nodeptr = nodeptr->next;
  }

  nodeptr = ses->variables;
  while(nodeptr) {
    if(ses && searchnode_list(master_table.main_session->variables,
			      nodeptr->left)) {
      nodeptr = nodeptr->next;
      continue;
    }
    fprintf(myfile, "%cvariable %c%s%c %c%s%c\n",
            tintin_char,
            text_start, nodeptr->left, text_end,
            text_start, nodeptr->right, text_end);
    nodeptr = nodeptr->next;
  }

  write_word_list(myfile, FALSE, ses);

  fclose(myfile);
  ses->print((const uchar *)"#COMMANDO-FILE WRITTEN.\n\r");
  return ses;
}
