/*********************************************************************/
/* file: path.c - stuff for the path feature                         */
/*                             HUT TINTIN                            */
/*                coded by Ossi Tulonen from TinTin III              */
/*                TinTin III coded by peter unold 1992               */
/*********************************************************************/
/* the path is implemented as a fix-sized queue. It gets a bit messy */
/* here and there, but it should work....                            */
/*********************************************************************/
/*
 * TinTin III code Copyright (c) 1992 Peter Unold
 * All changed code from TinTin III Copyright (c) 1992 1993 1994 Ossi tulonen
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
 
#include <stdio.h>
#include <string.h>
#ifndef NSTDLIBS
#include <unistd.h>
#endif

#include "structs.h"
#include "parse.h"
#include "path.h"
#include "screen.h"
#include "utility.h"

extern struct master_table_data master_table;
extern unsigned char tintin_char;
extern unsigned char text_start, text_end;

/*********************/
/* the #MARK command */
/*********************/
session *mark_command(const unsigned char *dummy, session *ses)
{
  ses->path_mark = 0;
  ses->path_now = 0;
  ses->path_length = 0;
  ses->print((const uchar *)"OK. MARK SET.\n\r");
  return(ses);
}

/*********************/
/* the #PATH command */
/*********************/
session *path_command(const unsigned char *dummy, session *ses)
{
  int pathpos, len, outlen;
  unsigned char mypath[MAX_PATH_LENGTH + 16] = "#PATH YOU WENT:";
  
  for(pathpos = ses->path_mark, len = ses->path_length, outlen = strlen((const char *)mypath);
      len;
      len--, pathpos=(pathpos+1)%MAX_PATH_LENGTH)
    mypath[outlen++] = ses->path[pathpos];
  
  mypath[outlen]     = '\n';
  mypath[outlen + 1] = '\r';
  mypath[outlen + 2] = '\0';
  ses->print(mypath);
  
  return(ses);
}

/* **********************
  the #savepath command
********************** */

session *comm_savepath(const unsigned char *arg, session *ses)
{
  unsigned char buf[BUFFER_SIZE], buf2[BUFFER_SIZE], path[MAX_PATH_LENGTH + 1];
  int pathpos, len, pos;
  
  arg = get_arg_stop_spaces(arg, buf2);
  if(!*buf2) {
    ses->print((const uchar *)"#Usage: savepath <alias to be>\n\r");
    return(ses);
  }
  for(pathpos = ses->path_mark, len = ses->path_length, pos = 0 ; len; 
      len--, pathpos = (pathpos + 1)%MAX_PATH_LENGTH)
    path[pos++] = ses->path[pathpos];
  path[pos] = '\0';
  
  if(!*path) {
    ses->print((const uchar *)"#ERROR - no path to put alias.\n\r");
    return(ses);
  }
  
  sprintf((char *)buf, "%calias %c%s%c %c%s%c", tintin_char,
	  text_start, buf2, text_end,
	  text_start, path, text_end);
  
  ses = parse_input(buf, ses);
  return(ses);
}

/***********************/
/* the #RETURN command */
/***********************/
session *return_command(const unsigned char *dummy, session *ses)
{
  if(ses->path_length) {
    unsigned char command[2]={'x', '\n'};
    
    command[0]=reverse_dir(ses->path[(ses->path_mark+(ses->path_length - 1))%MAX_PATH_LENGTH]);
    ses->path_length--;
    
    if(ses->logfile)
      fwrite(command, 1, 2, ses->logfile);

    if(ses->get_socket() >= 0)
      if(write(ses->get_socket(), command, 2)==-1)
	syserr((const uchar *)"write in send_to_mud");
  }
  return(ses);
}

/***********************/
/* the #UNPATH command */
/***********************/
session *unpath_command(const unsigned char *dummy, session *ses)
{
  if(ses->path_length) {
    ses->path_length--;
    ses->print((const uchar *)"OK. FORGOT THAT MOVE.\n\r");
  }

  return(ses);
}

/*********************************************************/
/* check to see if command=direction then insert in path */
/*********************************************************/
void check_insert_path(const uchar *command, session *ses)
{
  if(is_abrev((const uchar *)"north", command))
    insert_path('n', ses);
  else if(is_abrev((const uchar *)"east", command))
    insert_path('e', ses);
  else if(is_abrev((const uchar *)"south", command))
    insert_path('s', ses);
  else if(is_abrev((const uchar *)"west", command))
    insert_path('w', ses);
  else if(is_abrev((const uchar *)"up", command))
    insert_path('u', ses);
  else if(is_abrev((const uchar *)"down", command))
    insert_path('d', ses);
}

/***************************************************************************/
/* insert a direction in the path - first check if player went back though */
/* I know this gives problems with one-way-exits but......                 */
/***************************************************************************/
void insert_path(unsigned char dir, session *ses)
{
  if(ses->path_length && dir == reverse_dir(ses->path[(ses->path_mark+
    (ses->path_length-1))%MAX_PATH_LENGTH])) 
    ses->path_length--;             /*player went back*/
  else {
    if(ses->path_length==MAX_PATH_LENGTH)
      ses->path_mark=(ses->path_mark+1)%MAX_PATH_LENGTH;
    ses->path[(ses->path_mark+ses->path_length)%MAX_PATH_LENGTH]=dir;

    if(ses->path_length<MAX_PATH_LENGTH)
      ses->path_length++;
  }
}

/***************************************************/
/* return char containing reverse direction as dir */
/***************************************************/
unsigned char reverse_dir(unsigned char dir)
{
  switch(dir) {
  case 'n':
    return 's';
  case 'e':
    return 'w';
  case 's':
    return 'n';
  case 'w':
    return 'e';
  case 'u':
    return 'd';
  case 'd':
    return 'u';
  }
  return 'x';
}
