/*********************************************************************/
/* File: ticks.c - functions for the ticker stuff                    */
/*                             HUT TINTIN                            */
/*                coded by Ossi Tulonen from TinTin III              */
/*                TinTin III coded by peter unold 1992               */
/*********************************************************************/
/*
 * TinTin III code Copyright (c) 1992 Peter Unold
 * All changed code from TinTin III Copyright (c) 1992 1993 1994 Ossi tulonen
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
 
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>
#ifdef OSF1v3
typedef int clockid_t;
#endif
#include <time.h>
#ifndef NSTDLIBS
#include <stdlib.h>
#include <unistd.h>
#endif
#include <sys/types.h>
#ifdef TERMIOS
  #include <sys/termio.h>
#else
  #include <sys/ioctl.h>
#endif

#include "structs.h"
#include "parse.h"
#include "screen.h"
#include "ticks.h"
#include "utility.h"
#include "utils.h"

/* externs */
extern struct master_table_data master_table;
extern struct sgttyb original_state;
extern int default_chars_per_line;
extern int default_lines_in_scr;

/* local globals */
int window_interrupted;

/*********************/
/* the #tick command */
/*********************/
session *tick_command(const unsigned char *dummy, session *ses)
{
  if(ses->tickstatus) {
    unsigned char buf[100];
    sprintf((char *)buf, "THERE'S NOW %d SECONDS TO NEXT TICK.\n\r", ses->sec_to_tick);
    ses->print(buf);
  } else
    ses->print((const uchar *)"MY TICKER IS OFF! DUNNO SECONDS TO TICK\n\r");

  return(ses);
}

/************************/
/* the #tickoff command */
/************************/
session *tickoff_command(const unsigned char *dummy, session *ses)
{
  ses->tickstatus=FALSE;
  ses->print((const uchar *)"#TICKER IS NOW OFF.\n\r");

  return(ses);
}

/***********************/
/* the #tickon command */
/***********************/
session *tickon_command(const unsigned char *dummy, session *ses)
{
  ses->tickstatus=TRUE;
  ses->print((const uchar *)"#TICKER IS NOW ON.\n\r");

  return(ses);
}

/************************/
/* the #tickset command */
/************************/
session *tickset_command(const unsigned char *dummy, session *ses)
{
  ses->tick_start_time = time(NULL);
  /* we don't prompt! too many ticksets... */
  return(ses);
}

/*************************/
/* the #ticksize command */
/*************************/
session *ticksize_command(const unsigned char *arg, session *ses)
{
  if(*arg!='\0') {
    if(isdigit(*arg)) {
      ses->tick_size  = atoi((const char *)arg);
      ses->tick_start_time = time(NULL);
      ses->print((const uchar *)"#OK NEW TICKSIZE SET\n\r");
    } else
      ses->print((const uchar *)"#SPECIFY A NUMBER!!!!TRYING TO CRASH ME EH?\n\r");
  } else
    ses->print((const uchar *)"#SET THE TICK-SIZE TO WHAT?\n\r");
  
  return(ses);
}

void window_func(int i)
{
#if defined(HPUX) || defined(SUN)
  signal(SIGWINCH, window_func);
#endif

  window_interrupted = TRUE;
  get_screen_size();
}

session *comm_wait(const unsigned char *arg, session *ses)
{
  unsigned char time[BUFFER_SIZE], comm[BUFFER_SIZE];
  int num;
  struct waitcommdata *newcomm;

  if(!ses) {
    ses->print((const uchar *)"ERROR - #WAIT no active session.\n\r");
    return(ses);
  }
  arg = get_arg_stop_spaces(arg, time);
  arg = get_arg_with_spaces(arg, comm);
  if(!*time) {
    ses->print((const uchar *)"Following commands are waiting:\n\r");
    for(newcomm = ses->waitcomms; newcomm; newcomm = newcomm->next) {
      sprintf((char *)comm, "Wait time %3d: %s\n\r", newcomm->time, newcomm->comm);
      ses->print(comm);
    }
    return(ses);
  }

  num = string_to_number(time, ses);
  if(!*comm || num <= 0) {
    ses->print((const uchar *)"#ERROR - Usage: #wait <time> <text>\n\r");
    return(ses);
  }
  CREATE(newcomm, struct waitcommdata, 1);
  newcomm->comm = (uchar *)strdup((const char *)comm);
  newcomm->time = num;
  newcomm->next = ses->waitcomms;
  ses->waitcomms = newcomm;
  if(IS_SET(ses->message_flags, MESSAGE_WAIT))
    ses->print((const uchar *)"#OK. New waitcomm set.\n\r");
  return(ses);
}

/************************/
/* the #time command    */
/************************/
session *comm_time(const unsigned char *arg, session *ses)
{
  unsigned char buf[BUFFER_SIZE];
  int value;
  
  arg = get_arg_with_spaces(arg, buf);
  if(*buf)
    value = string_to_number(buf, ses);
  else
    value = -1;

  if(!ses->get_input()->change_time(value)) {
    ses->print((const uchar *)"#Time: Usage: #time [updating delay in secs]\n\r");
    ses->print((const uchar *)"#      -1 to turn off\n\r");
  }
  return(ses);
}
